package nl.amis.tourdefrance.model;

import java.io.IOException;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class ProcessTourStandings {

    private static final Pattern regexDistance = Pattern.compile("(Total distance covered:)*([0-9\\.]+)( km)+");

    public static void main(String[] args) throws MalformedURLException, IOException {
        // year: 2007-2011
        Stage stage = processStage(2007, 7, new HashMap<Integer, Rider>(), new Tour());
        for (Standing standing : stage.getStandings()) {
            System.out.println(standing.getRank() + "." + standing.getRider());
        }

    }

    public static void processTour(Tour tour) throws IOException, MalformedURLException {
        if (tour.isPrologue())
            tour.getStages().add(processStage(tour.getYear(), 0, tour.getRiders(), tour));

        for (int i = 1; i <= tour.getNumberOfStages(); i++) {
            tour.getStages().add(processStage(tour.getYear(), i, tour.getRiders(), tour));
        }
    }

    private static Stage processStage(int year, int stageSequence, Map<Integer, Rider> riders,
                                      Tour tour) throws java.io.IOException, java.net.MalformedURLException {
        // types of standing:
        //        OVERALL (Grand) rankings:  ITG (total), IMG (climbers), IJG (Youth), IPG (points), ETG (Team), ICG (Combativity)
        //            Stage (Etappe) rankings: ITE (total), IME (climbers), IJE (Youth), IPE (points), ETE (Team), ICE (Combativity)
        //
        String typeOfStanding = "ITE";

        URL stageStanding =
            new URL("http://www.letour.fr/" + year + "/TDF/LIVE/us/" + (stageSequence == 0 ? "0" : stageSequence +
                                                                        "00") + "/classement/" + typeOfStanding +
                    ".html");
        InputStream stream = stageStanding.openStream();
        Scanner scanner = new Scanner(stream);
        scanner.useDelimiter("</tr>");
        Stage stage = new Stage();
        stage.setSequence(stageSequence);
        stage.setTour(tour);
        boolean first = true;
        boolean firstStanding = true;
        while (scanner.hasNext()) {
            String entry = scanner.next();
            if (first) {
                first = false;
                Matcher regexMatcher = regexDistance.matcher(entry);
                if (regexMatcher.find()) {
                    String distanceString = regexMatcher.group();
                    stage.setTotalDistance(Float.parseFloat(distanceString.substring(0, distanceString.length() - 3)));
                    //                    System.out.println("Distance is: " + stage.getTotalDistance());
                }
            }
            if (!first) {
                String[] els = entry.split("/td>");
                if (els.length > 1) { // only the standing-entries have more than one td element
                    //                    for (int i = 0; i < els.length - 1; i++) {
                    //                        System.out.println(extractValue(els[i])+"***" +els[i]);
                    //                    }
                    Integer riderNumber = Integer.parseInt(extractValue(els[2]));

                    Rider rider = null;
                    if (riders.containsKey(riderNumber)) {
                        rider = riders.get(riderNumber);
                    } else {
                        rider = new Rider(extractValue(els[1]), riderNumber, extractValue(els[3]));
                        riders.put(riderNumber, rider);
                        rider.setTour(tour);
                    }
                    Standing standing =
                        new Standing(firstStanding ? 1 : (Integer.parseInt(extractValue(els[0]).replace(".", ""))),
                                     rider, extractValue(els[4]), extractValue(els[5]));
                    standing.setStage(stage);
                    firstStanding = false;
                    stage.getStandings().add(standing);
                }

            }

        } //while
        scanner.close();
        return stage;
    }

    private static String extractValue(String el) {
        String r = el.split("</")[0];
        if (r.lastIndexOf(">") > 0) {
            r = r.substring(r.lastIndexOf(">") + 1);
        }
        return r.split("<")[0];
    }
}

